/************************************************************************/
/*                            Prof v1.0                                 */
/*                     Author : Mohammed OUALI                          */
/*  A full description of the method is given in the paper entitled     */
/* "Cascaded Mutiple Classifers for Secondary Structure Prediction"     */
/*              by Mohammed OUALI and Ross D. KING                      */
/*                 (Submitted for publication)                          */
/*                                                                      */
/*   Copyright (C) 1999 Mohammed OUALI and University of Wales          */
/*             (Aberystwyth) Penglais, Aberystwyth                      */
/*                 Ceredigion SY23 3DB WALES                            */
/*                        U.K.                                          */
/*                                                                      */
/* This software can be copied and used freely providing it is not      */
/* resold in any form and its use is acknowledged.                      */
/* This software is provided by ``as is'' and any express or implied    */
/* warranties, including, but not limited to, the implied warranties of */
/* merchantability and fitness for a particular purpose are disclaimed. */
/* In no event shall the regents or contributors be liable for any      */
/* direct, indirect, incidental, special, exemplary, or consequential   */
/* damages (including, but not limited to, procurement of substitute    */
/* goods or services; loss of use, data, or profits; or business        */
/* interruption) however caused and on any theory of liability, whether */
/* in contract, strict liability, or tort (including negligence or      */
/* otherwise) arising in any way out of the use of this software, even  */
/* if advised of the possibility of such damage.                        */
/*                                                                      */
/************************************************************************/
#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include "Prof.h"
#include "gor.h"

void gorI(void)
{
int i, j, k, h, m     ;
int win_beg , win_end ;
int blank             ;
float denom           ;
char *gorI_dat        ;

FILE *fp       ;

void  read_param_gorI(FILE *fq, float *x) ;

if(verbose == 1) printf("gorI is started \n") ;

MLLC(gorI_dat, char, MAX_FIC) ;
MLLC(i_dira, float, 357)      ;
MLLC(i_dirb, float, 357)      ;
MLLC(i_dirc, float, 357)      ;

gor1 = (struct g1 *) malloc( (length_query+1)*sizeof (struct g1) ) ;

infa1 = alloue_2(nb_ali_seq, length_query+1) ; 
infb1 = alloue_2(nb_ali_seq, length_query+1) ; 
infc1 = alloue_2(nb_ali_seq, length_query+1) ;
pa1   = alloue_2(nb_ali_seq, length_query+1) ; 
pb1   = alloue_2(nb_ali_seq, length_query+1) ;
pc1   = alloue_2(nb_ali_seq, length_query+1) ; 

strcpy(gorI_dat, getenv("PROF_DIR")) ;
strcat(gorI_dat, "/gorI.param")      ;

if ((fp = fopen(gorI_dat , "r")) == NULL)
       {
       printf("Unable to open gorI table in function gorI ...\n");
       exit(1);
       }
read_param_gorI(fp, i_dira) ;
read_param_gorI(fp, i_dirb) ;
read_param_gorI(fp, i_dirc) ;
fclose(fp) ;

/****************************************************************************/
win_beg = -(WINSIZE/2) ;
win_end =  (WINSIZE/2) ;

for(i = 0 ; i < nb_ali_seq ; i++)
 {
 for(j = 0 ; j < size_prot[i] ; j++)
  {
   infa1[i][j] = infb1[i][j] = infc1[i][j] = 0. ;
   for(m= win_beg ; m <= win_end ; m++)
    {
      h = j+m ; k = m+OF_S ;
      if (h >= 0 && h < size_prot[i])
        {
          infa1[i][j] += i_dira(i_seq[i][h], k) ;
          infb1[i][j] += i_dirb(i_seq[i][h], k) ;
          infc1[i][j] += i_dirc(i_seq[i][h], k) ;
        }
    }
     pa1[i][j] = 1./ (1 + t1_a*exp(-infa1[i][j])) ;
     pb1[i][j] = 1./ (1 + t1_b*exp(-infb1[i][j])) ;
     pc1[i][j] = 1./ (1 + t1_c*exp(-infc1[i][j])) ;
     denom = pa1[i][j] + pb1[i][j] + pc1[i][j] ;
     pa1[i][j] /= denom ; pb1[i][j] /= denom ; pc1[i][j] /= denom ;
  }
 }
/****************************************************************************/
for(i = 0 ; i < nb_ali_seq ; i++)
 {
  rematch_align(infa1,  i) ;
  rematch_align(infb1,  i) ;
  rematch_align(infc1,  i) ;
  rematch_align(pa1,    i) ;
  rematch_align(pb1,    i) ;
  rematch_align(pc1,    i) ;
 }
/****************************************************************************/
for(i = 0 ; i < length_query ; i++)
 {
  gor1[i].info_a = 0. ; gor1[i].info_b = 0. ; gor1[i].info_c = 0. ;
  gor1[i].p_a    = 0. ; gor1[i].p_b    = 0. ; gor1[i].p_c    = 0. ;
  blank = 0 ;
  for(j = 0 ; j < nb_ali_seq ; j++)
   {
    if(seq[0][i] == '-' || seq[0][i] == '.') break ;
    else
     {
      if(seq[j][i] == '-' || seq[j][i] == '.') blank++ ;
      else
       {
        gor1[i].info_a += infa1[j][i] ; gor1[i].info_b += infb1[j][i] ; gor1[i].info_c += infc1[j][i] ;
        gor1[i].p_a    += pa1[j][i]   ; gor1[i].p_b    += pb1[j][i]   ; gor1[i].p_c    += pc1[j][i]   ;
       }
     }
   }
  gor1[i].info_a /= (float) (nb_ali_seq-blank) ;
  gor1[i].info_b /= (float) (nb_ali_seq-blank) ;
  gor1[i].info_c /= (float) (nb_ali_seq-blank) ;
  gor1[i].p_a    /= (float) (nb_ali_seq-blank) ;
  gor1[i].p_b    /= (float) (nb_ali_seq-blank) ;
  gor1[i].p_c    /= (float) (nb_ali_seq-blank) ;
 }
/****************************************************************************/
/*
for(i=0 ; i <  length_query ; i++)
   {
   printf("%f %f %f %f %f %f \n",
   gor1[i].info_a, gor1[i].info_b, gor1[i].info_c, gor1[i].p_a, gor1[i].p_b, gor1[i].p_c) ;
   }
*/



/* On fait free des i_dir apres gorIII car on a besoin de l'element 
 * central 
*/

if(verbose == 1) printf("gorI is completed ...\n") ;


return ;
}

