/************************************************************************/
/*                            Prof v1.0                                 */
/*                     Author : Mohammed OUALI                          */
/*  A full description of the method is given in the paper entitled     */
/* "Cascaded Mutiple Classifers for Secondary Structure Prediction"     */
/*              by Mohammed OUALI and Ross D. KING                      */
/*                 (Submitted for publication)                          */
/*                                                                      */
/*   Copyright (C) 1999 Mohammed OUALI and University of Wales          */
/*             (Aberystwyth) Penglais, Aberystwyth                      */
/*                 Ceredigion SY23 3DB WALES                            */
/*                        U.K.                                          */
/*                                                                      */
/* This software can be copied and used freely providing it is not      */
/* resold in any form and its use is acknowledged.                      */
/* This software is provided by ``as is'' and any express or implied    */
/* warranties, including, but not limited to, the implied warranties of */
/* merchantability and fitness for a particular purpose are disclaimed. */
/* In no event shall the regents or contributors be liable for any      */
/* direct, indirect, incidental, special, exemplary, or consequential   */
/* damages (including, but not limited to, procurement of substitute    */
/* goods or services; loss of use, data, or profits; or business        */
/* interruption) however caused and on any theory of liability, whether */
/* in contract, strict liability, or tort (including negligence or      */
/* otherwise) arising in any way out of the use of this software, even  */
/* if advised of the possibility of such damage.                        */
/*                                                                      */
/************************************************************************/
#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include "Prof.h"
#include "gor.h"

void gorIII(void)
{
int i , j , k , h , m ;
int win_beg , win_end ;
int blank             ;
float denom           ;
char *gorIII_dat      ;

FILE *fp              ;


void  read_param_gorIII(FILE *fq, float *x) ;

if(verbose == 1) printf("gorIII is started \n") ;

MLLC(gorIII_dat, char, MAX_FIC) ;
MLLC(i_paira, float, 7497)      ;
MLLC(i_pairb, float, 7497)      ;
MLLC(i_pairc, float, 7497)      ;

gor3 = (struct g1 *) malloc( (length_query+1)*sizeof (struct g1) )  ;

infa3 = alloue_2(nb_ali_seq, length_query+1) ;
infb3 = alloue_2(nb_ali_seq, length_query+1) ;
infc3 = alloue_2(nb_ali_seq, length_query+1) ;
pa3   = alloue_2(nb_ali_seq, length_query+1) ;
pb3   = alloue_2(nb_ali_seq, length_query+1) ;
pc3   = alloue_2(nb_ali_seq, length_query+1) ;

strcpy(gorIII_dat, getenv("PROF_DIR")) ;
strcat(gorIII_dat, "/gorIII.param")      ;

if ((fp = fopen(gorIII_dat , "r")) == NULL)
       {
       printf("Unable to open gorIII table in function gorIII ...\n");
       exit(1);
       }
read_param_gorIII(fp, i_paira) ;
read_param_gorIII(fp, i_pairb) ;
read_param_gorIII(fp, i_pairc) ;
fclose(fp) ;
/****************************************************************************/
win_beg = -(WINSIZE/2) ;
win_end =  (WINSIZE/2) ;
for(i = 0 ; i < nb_ali_seq ; i++)
 {
 for(j = 0 ; j < size_prot[i] ; j++)
  {
   infa3[i][j] = i_dira(i_seq[i][j] , OF_S) ;
   infb3[i][j] = i_dirb(i_seq[i][j] , OF_S) ;
   infc3[i][j] = i_dirc(i_seq[i][j] , OF_S) ;
   for(m= win_beg ; m <= win_end ; m++)
    {
      h = j+m ; k = m+OF_S ;
      if (h >= 0 && h < size_prot[i] && m != 0)
        {
          infa3[i][j] += i_paira(i_seq[i][j], i_seq[i][h], k) ;
          infb3[i][j] += i_pairb(i_seq[i][j], i_seq[i][h], k) ;
          infc3[i][j] += i_pairc(i_seq[i][j], i_seq[i][h], k) ;
        }
    }
     pa3[i][j] = 1./ (1 + t1_a*exp(-infa3[i][j])) ;
     pb3[i][j] = 1./ (1 + t1_b*exp(-infb3[i][j])) ;
     pc3[i][j] = 1./ (1 + t1_c*exp(-infc3[i][j])) ;
     denom = pa3[i][j] + pb3[i][j] + pc3[i][j] ;
     pa3[i][j] /= denom ; pb3[i][j] /= denom ; pc3[i][j] /= denom ;
  }
 }
/****************************************************************************/
for(i = 0 ; i < nb_ali_seq ; i++)
 {
  rematch_align(infa3,   i) ;
  rematch_align(infb3,   i) ;
  rematch_align(infc3,   i) ;
  rematch_align(pa3,     i) ;
  rematch_align(pb3,     i) ;
  rematch_align(pc3,     i) ;
 }
/****************************************************************************/
for(i = 0 ; i < length_query ; i++)
 {
  gor3[i].info_a = 0. ; gor3[i].info_b = 0. ; gor3[i].info_c = 0. ;
  gor3[i].p_a    = 0. ; gor3[i].p_b    = 0. ; gor3[i].p_c    = 0. ;
  blank = 0 ;
  for(j = 0 ; j < nb_ali_seq ; j++)
   {
    if(seq[0][i] == '-' || seq[0][i] == '.') break ;
    else
     {
      if(seq[j][i] == '-' || seq[j][i] == '.') blank++ ;
      else
       {
        gor3[i].info_a += infa3[j][i] ; gor3[i].info_b += infb3[j][i] ; gor3[i].info_c += infc3[j][i] ;
        gor3[i].p_a    += pa3[j][i]   ; gor3[i].p_b    += pb3[j][i]   ; gor3[i].p_c    += pc3[j][i]   ;
       }
     }
   }
  gor3[i].info_a /= (float) (nb_ali_seq-blank) ;
  gor3[i].info_b /= (float) (nb_ali_seq-blank) ;
  gor3[i].info_c /= (float) (nb_ali_seq-blank) ;
  gor3[i].p_a    /= (float) (nb_ali_seq-blank) ;
  gor3[i].p_b    /= (float) (nb_ali_seq-blank) ;
  gor3[i].p_c    /= (float) (nb_ali_seq-blank) ;
 }
/****************************************************************************/

/*
printf("+++++++++++++++++++++++++++++++++++++++++++++++\n") ;
for(i=0 ; i <  length_query ; i++)
   {
   printf("%f %f %f %f %f %f \n",
   gor3[i].info_a, gor3[i].info_b, gor3[i].info_c, gor3[i].p_a, gor3[i].p_b, gor3[i].p_c) ;
   }
*/



free(i_dira)  ;
free(i_dirb)  ;
free(i_dirc)  ;
free(i_paira) ;
free(i_pairb) ;
free(i_pairc) ;
if(verbose == 1) printf("gorIII is completed ...\n") ;

return ;
}

