/************************************************************************/
/*                            Prof v1.0                                 */
/*                     Author : Mohammed OUALI                          */
/*  A full description of the method is given in the paper entitled     */
/* "Cascaded Mutiple Classifers for Secondary Structure Prediction"     */
/*              by Mohammed OUALI and Ross D. KING                      */
/*                 (Submitted for publication)                          */
/*                                                                      */
/*   Copyright (C) 1999 Mohammed OUALI and University of Wales          */
/*             (Aberystwyth) Penglais, Aberystwyth                      */
/*                 Ceredigion SY23 3DB WALES                            */
/*                        U.K.                                          */
/*                                                                      */
/* This software can be copied and used freely providing it is not      */
/* resold in any form and its use is acknowledged.                      */
/* This software is provided by ``as is'' and any express or implied    */
/* warranties, including, but not limited to, the implied warranties of */
/* merchantability and fitness for a particular purpose are disclaimed. */
/* In no event shall the regents or contributors be liable for any      */
/* direct, indirect, incidental, special, exemplary, or consequential   */
/* damages (including, but not limited to, procurement of substitute    */
/* goods or services; loss of use, data, or profits; or business        */
/* interruption) however caused and on any theory of liability, whether */
/* in contract, strict liability, or tort (including negligence or      */
/* otherwise) arising in any way out of the use of this software, even  */
/* if advised of the possibility of such damage.                        */
/*                                                                      */
/************************************************************************/
#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include "Prof.h"
#include "gor.h"

void gorIV(void)
{
int i , j , k , l , h , g ,  m  , n ;
int win_beg , win_end ;
int blank             ;
float denom           ;
char *gorIV_dat       ;
FILE *fp              ;

void  read_param_gorIV(FILE *fq, float *x, int flg) ;

if(verbose == 1) printf("gorIV is started \n") ;

MLLC(gorIV_dat, char, MAX_FIC) ;

gor4 = (struct g2 *) malloc( (length_query+1)*sizeof (struct g2) )  ;

ida4 = alloue_2(nb_ali_seq, length_query+1) ;
idb4 = alloue_2(nb_ali_seq, length_query+1) ;
idc4 = alloue_2(nb_ali_seq, length_query+1) ;
pa4  = alloue_2(nb_ali_seq, length_query+1) ;
pb4  = alloue_2(nb_ali_seq, length_query+1) ;
pc4  = alloue_2(nb_ali_seq, length_query+1) ;


strcpy(gorIV_dat, getenv("PROF_DIR")) ;
strcat(gorIV_dat, "/gorIV.param")      ;

MLLC(ratio_s_ra, float, 357)   ;
MLLC(ratio_s_rb, float, 357)   ;
MLLC(ratio_s_rc, float, 357)   ;

MLLC(ratio_pair_paira, float, 127449)   ;
MLLC(ratio_pair_pairb, float, 127449)   ;
MLLC(ratio_pair_pairc, float, 127449)   ;

if ((fp = fopen(gorIV_dat , "r")) == NULL)
       {
       printf("Unable to open gorIV table in function gorIV ...\n");
       exit(1);
       }

read_param_gorIV(fp, ratio_s_ra, 0) ;
read_param_gorIV(fp, ratio_s_rb, 0) ;
read_param_gorIV(fp, ratio_s_rc, 0) ;

read_param_gorIV(fp, ratio_pair_paira, 1) ;
read_param_gorIV(fp, ratio_pair_pairb, 1) ;
read_param_gorIV(fp, ratio_pair_pairc, 1) ;

fclose(fp) ;

/***************************************************************************/
win_beg = -(WINSIZE/2) ;
win_end =  (WINSIZE/2) ;
for(i = 0 ; i < nb_ali_seq ; i++)
 {
 for(j = 0 ; j < size_prot[i] ; j++)
  {
   ida4[i][j] = idb4[i][j] = idc4[i][j] = 0. ;
/*-------------------------------------------------------*/
   for(m= win_beg ; m <= win_end ; m++)
    {
     h = j+m ; k = m+OF_S ;
     if(h >= 0 && h < size_prot[i])
      {
      ida4[i][j] += const_1*ratio_s_ra(i_seq[i][h] , k) ;
      idb4[i][j] += const_1*ratio_s_rb(i_seq[i][h] , k) ;
      idc4[i][j] += const_1*ratio_s_rc(i_seq[i][h] , k) ;
      }
    }
/*-------------------------------------------------------*/
   for(m= win_beg ; m <  win_end ; m++)
    {
    h = j+m ; k = m+OF_S ;
    if(h >= 0 && h < size_prot[i])
     {
      for(n = m+1 ; n <= win_end ; n++)
       {
        g = j+n ; l = n+OF_S ;
        if(g >= 0 && g <  size_prot[i])
         {
          ida4[i][j] += const_2*ratio_pair_paira(i_seq[i][h] , k , i_seq[i][g] , l ) ;
          idb4[i][j] += const_2*ratio_pair_pairb(i_seq[i][h] , k , i_seq[i][g] , l ) ;
          idc4[i][j] += const_2*ratio_pair_pairc(i_seq[i][h] , k , i_seq[i][g] , l ) ;
         }
       }
     }
    }
/*-------------------------------------------------------*/
   pa4[i][j] = 1. / (1+exp(-ida4[i][j])) ;
   pb4[i][j] = 1. / (1+exp(-idb4[i][j])) ;
   pc4[i][j] = 1. / (1+exp(-idc4[i][j])) ;
   denom = pa4[i][j] + pb4[i][j] + pc4[i][j] ;
   pa4[i][j] /= denom ; pb4[i][j] /= denom ; pc4[i][j] /= denom ;
  }
 }
/***************************************************************************/
for(i = 0 ; i < nb_ali_seq ; i++)
 {
  rematch_align(ida4 ,   i) ;
  rematch_align(idb4 ,   i) ;
  rematch_align(idc4 ,   i) ;
  rematch_align(pa4  ,   i) ;
  rematch_align(pb4  ,   i) ;
  rematch_align(pc4  ,   i) ;
 }
/***************************************************************************/
for(i = 0 ; i < length_query ; i++)
 {
  gor4[i].id_a = 0. ; gor4[i].id_b = 0. ; gor4[i].id_c = 0. ;
  gor4[i].p_a  = 0. ; gor4[i].p_b  = 0. ; gor4[i].p_c  = 0. ;
  blank = 0 ;
  for(j = 0 ; j < nb_ali_seq ; j++)
   {
    if(seq[0][i] == '-' || seq[0][i] == '.') break ;
    else
     {
      if(seq[j][i] == '-' || seq[j][i] == '.') blank++ ;
      else
       {
        gor4[i].id_a += ida4[j][i] ; gor4[i].id_b += idb4[j][i] ; gor4[i].id_c += idc4[j][i] ;
        gor4[i].p_a  += pa4[j][i]  ; gor4[i].p_b  += pb4[j][i]  ; gor4[i].p_c  += pc4[j][i]  ;
       }
     }
   }
  gor4[i].id_a /= (float) (nb_ali_seq-blank) ;
  gor4[i].id_b /= (float) (nb_ali_seq-blank) ;
  gor4[i].id_c /= (float) (nb_ali_seq-blank) ;
  gor4[i].p_a  /= (float) (nb_ali_seq-blank) ;
  gor4[i].p_b  /= (float) (nb_ali_seq-blank) ;
  gor4[i].p_c  /= (float) (nb_ali_seq-blank) ;
 }
/***************************************************************************/

/*
printf("+++++++++++++++++++++++++++++++++++++++++++++++\n") ;
for(i=0 ; i <  length_query ; i++)
   {
   printf("%f %f %f %f %f %f \n",
   gor4[i].id_a, gor4[i].id_b, gor4[i].id_c, gor4[i].p_a, gor4[i].p_b, gor4[i].p_c) ;
   }
*/

free(ratio_s_ra)       ;
free(ratio_s_rb)       ;
free(ratio_s_rc)       ;
free(ratio_pair_paira) ;
free(ratio_pair_pairb) ;
free(ratio_pair_pairc) ;
if(verbose) printf("gorIV is completed ...\n") ;

return ;
}

