/************************************************************************/
/*                            Prof v1.0                                 */
/*                     Author : Mohammed OUALI                          */
/*  A full description of the method is given in the paper entitled     */
/* "Cascaded Mutiple Classifers for Secondary Structure Prediction"     */
/*              by Mohammed OUALI and Ross D. KING                      */
/*                 (Submitted for publication)                          */
/*                                                                      */
/*   Copyright (C) 1999 Mohammed OUALI and University of Wales          */
/*             (Aberystwyth) Penglais, Aberystwyth                      */
/*                 Ceredigion SY23 3DB WALES                            */
/*                        U.K.                                          */
/*                                                                      */
/* This software can be copied and used freely providing it is not      */
/* resold in any form and its use is acknowledged.                      */
/* This software is provided by ``as is'' and any express or implied    */
/* warranties, including, but not limited to, the implied warranties of */
/* merchantability and fitness for a particular purpose are disclaimed. */
/* In no event shall the regents or contributors be liable for any      */
/* direct, indirect, incidental, special, exemplary, or consequential   */
/* damages (including, but not limited to, procurement of substitute    */
/* goods or services; loss of use, data, or profits; or business        */
/* interruption) however caused and on any theory of liability, whether */
/* in contract, strict liability, or tort (including negligence or      */
/* otherwise) arising in any way out of the use of this software, even  */
/* if advised of the possibility of such damage.                        */
/*                                                                      */
/************************************************************************/
#include<stdio.h>
#include<stdlib.h>
#include "math.h"
#include "Prof.h"
#include "gor.h"

void linear_gor(void) 
{
int i, j ;
float blank ;
float norm, dxa, dxb, dxc ;

float poida[16] =  {-0.961702,  -0.568220,  1.124843,  0.897029,  3.977173, -2.274017, -0.175332, -0.053663,
                    -0.125009,   2.038640, -0.444611,  1.106482, -0.493271, -0.729580, -3.628578, -1.570679} ;
float poidb[16] =  {-1.852991,   0.998099, -0.387484, -0.186134, -2.197205,  3.878479,  0.025999, -0.137490,
                    -0.031839,  -1.073898,  2.003700, -0.844282,  0.399382, -0.350739,  0.187820,  0.885193} ;
float poidc[16] =  {-0.268308,  -0.163319, -0.764597, -0.693907, -2.136810, -0.574181,  0.142741,  0.143311,
                     0.136277,  -1.131216, -0.965297, -0.434911,  0.178130,  0.908295,  3.193970,  0.831955} ;



MLLC(pa_l, float, length_query) ;
MLLC(pb_l, float, length_query) ;
MLLC(pc_l, float, length_query) ;

if(verbose == 1) printf("gor with linear discrimination is started ... \n") ;

for(j = 0 ; j < length_query ; j++)
 {
 pa_l[j] = 0. ; pb_l[j] = 0. ; pc_l[j] = 0. ;
 blank = 0. ;
 for(i = 0 ; i < nb_ali_seq ; i++)
  {
   if(seq[0][j] == '-' || seq[0][j] == '.') break ;
   else
   {
   if(seq[i][j] == '-' || seq[i][j] == '.') blank++ ;
   else
    {
     dxa = poida[0] +poida[1]*infa1[i][j] +poida[2]*infb1[i][j] +poida[3]*infc1[i][j] +
           poida[4]*pa1[i][j] +poida[5]*pb1[i][j] +
           poida[6]*infa3[i][j] +poida[7]*infb3[i][j] +poida[8]*infc3[i][j] +
           poida[9]*pa3[i][j] +poida[10]*pb3[i][j] +
           poida[11]*ida4[i][j] +poida[12]*idb4[i][j] +poida[13]*idc4[i][j] +
           poida[14]*pa4[i][j] +poida[15]*pb4[i][j] ;

     dxb = poidb[0] +poidb[1]*infa1[i][j] +poidb[2]*infb1[i][j] +poidb[3]*infc1[i][j] +
           poidb[4]*pa1[i][j] +poidb[5]*pb1[i][j] +
           poidb[6]*infa3[i][j] +poidb[7]*infb3[i][j] +poidb[8]*infc3[i][j] +
           poidb[9]*pa3[i][j] +poidb[10]*pb3[i][j] +
           poidb[11]*ida4[i][j] +poidb[12]*idb4[i][j] +poidb[13]*idc4[i][j] +
           poidb[14]*pa4[i][j] +poidb[15]*pb4[i][j] ;

     dxc = poidc[0] +poidc[1]*infa1[i][j] +poidc[2]*infb1[i][j] +poidc[3]*infc1[i][j] +
           poidc[4]*pa1[i][j] +poidc[5]*pb1[i][j] +
           poidc[6]*infa3[i][j] +poidc[7]*infb3[i][j] +poidc[8]*infc3[i][j] +
           poidc[9]*pa3[i][j] +poidc[10]*pb3[i][j] +
           poidc[11]*ida4[i][j] +poidc[12]*idb4[i][j] +poidc[13]*idc4[i][j] +
           poidc[14]*pa4[i][j] +poidc[15]*pb4[i][j] ;

     pa_l[j] += 1./(1.+exp(-dxa)) ;
     pb_l[j] += 1./(1.+exp(-dxb)) ;
     pc_l[j] += 1./(1.+exp(-dxc)) ;
    }
   }
  }
  pa_l[j] /= (float) (nb_ali_seq-blank) ;
  pb_l[j] /= (float) (nb_ali_seq-blank) ;
  pc_l[j] /= (float) (nb_ali_seq-blank) ;
  if(pa_l[j] != 0. && pb_l[j] != 0. && pc_l[j] != 0.)
   {
    norm = pa_l[j] + pb_l[j] +pc_l[j] ;
    pa_l[j] /= norm ;
    pb_l[j] /= norm ;
    pc_l[j] /= norm ;
   }
 }

/*
for(i = 0 ; i < length_query ; i++)
 printf("%c %f %f %f \n", seq[0][i], pa_l[i], pb_l[i], pc_l[i]) ;
printf("-----------------------------------\n") ;
*/

/** Free of non needed tables **/

for(i = 0; i <  nb_ali_seq ; i++)
{
 free(infa1[i]) ; free(pa1[i]) ;
 free(infb1[i]) ; free(pb1[i]) ;
 free(infc1[i]) ; free(pc1[i]) ;
 free(infa3[i]) ; free(pa3[i]) ;
 free(infb3[i]) ; free(pb3[i]) ;
 free(infc3[i]) ; free(pc3[i]) ;
 free(ida4[i])  ; free(pa4[i]) ;
 free(idb4[i])  ; free(pb4[i]) ;
 free(idc4[i])  ; free(pc4[i]) ;
}
 free(infa1) ; free(pa1) ;
 free(infb1) ; free(pb1) ;
 free(infc1) ; free(pc1) ;
 free(infa3) ; free(pa3) ;
 free(infb3) ; free(pb3) ;
 free(infc3) ; free(pc3) ;
 free(ida4)  ; free(pa4) ;
 free(idb4)  ; free(pb4) ;
 free(idc4)  ; free(pc4) ;

if(verbose == 1) printf("gor with linear discrimination is completed ...\n") ;

return ;
}
