/************************************************************************/
/*                            Prof v1.0                                 */
/*                     Author : Mohammed OUALI                          */
/*  A full description of the method is given in the paper entitled     */
/* "Cascaded Mutiple Classifers for Secondary Structure Prediction"     */
/*              by Mohammed OUALI and Ross D. KING                      */
/*                 (Submitted for publication)                          */
/*                                                                      */
/*   Copyright (C) 1999 Mohammed OUALI and University of Wales          */
/*             (Aberystwyth) Penglais, Aberystwyth                      */
/*                 Ceredigion SY23 3DB WALES                            */
/*                        U.K.                                          */
/*                                                                      */
/* This software can be copied and used freely providing it is not      */
/* resold in any form and its use is acknowledged.                      */
/* This software is provided by ``as is'' and any express or implied    */
/* warranties, including, but not limited to, the implied warranties of */
/* merchantability and fitness for a particular purpose are disclaimed. */
/* In no event shall the regents or contributors be liable for any      */
/* direct, indirect, incidental, special, exemplary, or consequential   */
/* damages (including, but not limited to, procurement of substitute    */
/* goods or services; loss of use, data, or profits; or business        */
/* interruption) however caused and on any theory of liability, whether */
/* in contract, strict liability, or tort (including negligence or      */
/* otherwise) arising in any way out of the use of this software, even  */
/* if advised of the possibility of such damage.                        */
/*                                                                      */
/************************************************************************/
#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<ctype.h>
#include "Prof.h"

main(int argc, char **argv)
{
 char *arg       ;
 int count , err ;

 err         = 0    ;
 mode        = 0    ; /* automatic mode */
 verbose     = 0    ; /* not verbose    */
 del_int     = 0    ; /* delete intermediate file if del_int == 1*/
 check_pred  = 0    ; /* analyse the different prediction when set to 1 */
 casp        = 0    ; /* casp output format */
 file_seq    = NULL ;
 file_dat    = "nr" ;
 file_out    = NULL ;
 file_clu    = NULL ;
 file_psi    = NULL ;

 if (argc == 1) {
                  usage();
                  exit(0);
                }
 for(count=1; count < argc; count++)
  {
   arg = argv[count] ;
   if(!strcmp(arg,"-v")) { verbose    = 1                   ; continue ; } 
   if(!strcmp(arg,"-d")) { del_int    = 1                   ; continue ; }
   if(!strcmp(arg,"-A")) { check_pred = 1                   ; continue ; }
   if(!strcmp(arg,"-c")) { casp       = 1                   ; continue ; }
   if(!strcmp(arg,"-m")) { mode       = atoi(argv[++count]) ; continue ; } 
   if(!strcmp(arg,"-i")) { file_seq   = argv[++count]       ; continue ; }
   if(!strcmp(arg,"-a")) { file_clu   = argv[++count]       ; continue ; }
   if(!strcmp(arg,"-p")) { file_psi   = argv[++count]       ; continue ; }
   if(!strcmp(arg,"-b")) { file_dat   = argv[++count]       ; continue ; }
   if(!strcmp(arg,"-o")) { file_out   = argv[++count]       ; continue ; }
   printf("bad option %s\n",arg) ; err = 1 ;
  }
 if(!file_out) { printf("file of secondary structure output not found\n") ; err = 1 ; }
 if(mode == 1) {
                if(!file_clu) err = 1 ; 
                if(!file_psi) err = 1 ; 
               }
 else          {
                if(!file_seq) err = 1; 
               }
 if(err) { usage() ; exit(0) ; }


 if(mode == 0)
  {
   read_seq()          ;
   call_psiblast()     ;
   call_trimmer()      ;
   call_clustalw()     ;
   read_align()        ;
   translate_align()   ;
   compute_profils()   ;
   gorI()              ;
   gorIII()            ;
   gorIV()             ;
   linear_gor()        ;
   quadratic_gor()     ;
   write_all_gor()     ;
   call_gor_nets()     ;
   call_profils_nets() ;
   call_phd_nets()     ;
   call_psi_nets()     ;
   write_step2()       ;
   net_line_step3()    ;
   net_step4()         ;
  }
 else
  {
   read_align()        ;
   translate_align()   ;
   compute_profils()   ;
   gorI()              ;
   gorIII()            ;
   gorIV()             ;
   linear_gor()        ;
   quadratic_gor()     ;
   write_all_gor()     ;
   call_gor_nets()     ;
   call_profils_nets() ;
   call_phd_nets()     ;
   call_psi_nets()     ;
   write_step2()       ;
   net_line_step3()    ;
   net_step4()         ;
  }
if(del_int == 1) del_int_files()        ;

}
