/************************************************************************/
/*                            Prof v1.0                                 */
/*                     Author : Mohammed OUALI                          */
/*  A full description of the method is given in the paper entitled     */
/* "Cascaded Mutiple Classifers for Secondary Structure Prediction"     */
/*              by Mohammed OUALI and Ross D. KING                      */
/*                 (Submitted for publication)                          */
/*                                                                      */
/*   Copyright (C) 1999 Mohammed OUALI and University of Wales          */
/*             (Aberystwyth) Penglais, Aberystwyth                      */
/*                 Ceredigion SY23 3DB WALES                            */
/*                        U.K.                                          */
/*                                                                      */
/* This software can be copied and used freely providing it is not      */
/* resold in any form and its use is acknowledged.                      */
/* This software is provided by ``as is'' and any express or implied    */
/* warranties, including, but not limited to, the implied warranties of */
/* merchantability and fitness for a particular purpose are disclaimed. */
/* In no event shall the regents or contributors be liable for any      */
/* direct, indirect, incidental, special, exemplary, or consequential   */
/* damages (including, but not limited to, procurement of substitute    */
/* goods or services; loss of use, data, or profits; or business        */
/* interruption) however caused and on any theory of liability, whether */
/* in contract, strict liability, or tort (including negligence or      */
/* otherwise) arising in any way out of the use of this software, even  */
/* if advised of the possibility of such damage.                        */
/*                                                                      */
/************************************************************************/
#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include "Prof.h"
#include "All_net.h"

void net_line_step3(void)
{
int i , j , k , u , m                ;
int pred_a , pred_b , pred_c         ;
int fh , fe , fq , fd , fr           ;
char v[3]                            ;
float c_a, c_b, c_c                  ;
float dxa, dxb, dxc                  ;
float pa, pb, pc                     ;
float aa0 , ba0 , ab0 , bb0          ;
float frac_alpha , frac_beta, frac_h ;
float frac_e, frac_q, frac_d, frac_r ;
float coeff_hydro[21]                ;
float *hydro_alpha , *hydro_beta     ;
float **attrib                       ;
float *netInput,  *netOutput         ;

FILE *fp            ;

/**/
float poida[24] =  { 0.581056, -0.466000, -0.363230, -0.144003, -0.426292, -0.572549,
                    -0.079149,  0.218213,  0.013975,  0.300631,  0.102769,  0.012951,
                     0.191256, -0.188257, -0.296317, -0.303450, -0.268996, -0.274100,
                     1.333314, -0.267624, -0.119093,  1.210402, -0.347155, -0.047554} ;

float poidb[24] =  { -0.128842,  1.072453, -0.075306,  0.403921,  0.745892,  0.532811,
                      0.116499, -0.262509, -0.207982, -0.071782,  0.379128, -0.110152,
                      0.107736,  0.795807, -0.090126,  0.031147, -0.202665,  0.259395, 
                     -0.714696,  1.452761, -0.557837, -0.540444,  1.351963, -0.529318} ;

float poidc[24] =  { -0.445747, -0.304377,  0.385205, -0.143733, -0.117955,  0.161943,
                     -0.006856, -0.021140,  0.129252, -0.227160, -0.353384,  0.063344,
                     -0.249297, -0.370626,  0.333804,  0.257490,  0.385527,  0.074605, 
                     -0.736712, -0.746482,  0.490446, -0.742817, -0.604557,  0.405251} ; 


c_a = -1.006346 ; c_b = -2.701530 ; c_c = 0.099119 ;
/**/

/**/
coeff_hydro[0]  =  0.62 ; coeff_hydro[1]  =  0.29 ;  coeff_hydro[2]  = -0.90 ;
coeff_hydro[3]  = -0.74 ; coeff_hydro[4]  =  1.20 ;  coeff_hydro[5]  =  0.48 ;
coeff_hydro[6]  = -0.40 ; coeff_hydro[7]  =  1.40 ;  coeff_hydro[8]  = -1.50 ;
coeff_hydro[9]  =  1.10 ; coeff_hydro[10] =  0.64 ;  coeff_hydro[11] = -0.78 ;
coeff_hydro[12] =  0.12 ; coeff_hydro[13] = -0.85 ;  coeff_hydro[14] = -2.50 ;
coeff_hydro[15] = -0.18 ; coeff_hydro[16] = -0.05 ;  coeff_hydro[17] =  1.10 ;
coeff_hydro[18] =  0.81 ; coeff_hydro[19] =  0.26 ;  coeff_hydro[20] =  0.00 ;
/**/

MLLC(netInput  , float, (step3_netREC.NoOfInput)   ) ;
MLLC(netOutput , float, (step3_netREC.NoOfOutput)  ) ;
MLLC(hydro_alpha, float, size_prot[0])               ;
MLLC(hydro_beta, float, size_prot[0])                ;
attrib = alloue_2( 24 , size_prot[0])                ;
step3 = (struct net_stage3 *) malloc( (size_prot[0])*sizeof (struct net_stage3) ) ;

if(verbose == 1) printf("neural networks of step 3 and subsquent computations are started ...\n") ;


if ((fp=fopen(name_step2, "r")) == NULL)
       {
       printf("Unable to open file for reading in function net_line_step3 ... \n") ;
       exit(1);
       }

for(i = 0 ; i < size_prot[0] ; i++)
 {
  fscanf(fp, "%f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f\n",
             &attrib [0][i], &attrib [1][i], &attrib [2][i], &attrib [3][i], &attrib [4][i],
             &attrib [5][i], &attrib [6][i], &attrib [7][i], &attrib [8][i], &attrib [9][i],
             &attrib[10][i], &attrib[11][i], &attrib[12][i], &attrib[13][i], &attrib[14][i],
             &attrib[15][i], &attrib[16][i], &attrib[17][i], &attrib[18][i], &attrib[19][i],
             &attrib[20][i], &attrib[21][i], &attrib[22][i], &attrib[23][i]) ;             
 }

fclose(fp) ;

for(i = 0 ; i < size_prot[0] ; i++)
  {
   dxa = c_a ;
   dxb = c_b ;
   dxc = c_c ;
   for(j =0 ; j < 24 ; j++)
    {
     dxa +=  poida[j]* attrib[j][i] ;
     dxb +=  poidb[j]* attrib[j][i] ;
     dxc +=  poidc[j]* attrib[j][i] ;
    }
    step3[i].lpa = 1./(1. + exp(-dxa)) ;
    step3[i].lpb = 1./(1. + exp(-dxb)) ;
    step3[i].lpc = 1./(1. + exp(-dxc)) ;
  }

for(i = 0 ; i <  size_prot[0] ; i++)
 {
  u = 0 ;
  for(j=i-6 ; j <= i+6 ; j++)
   {
    if(j >= 0 && j < size_prot[0])
     {
     for(k = 0 ; k < 24 ; k++) { netInput[u] = attrib[k][j] ; u++ ; }
     }
    else
     {
     for(k = 0 ; k < 24 ; k++) { netInput[u] = 0. ; u++ ; }
     }
   }
  step3_net(netInput, netOutput, 0)     ;
  step3[i].nnb_pa = netOutput[0]        ;
  step3[i].nnb_pb = netOutput[1]        ;
  step3[i].nnb_pc = netOutput[2]        ;
  step3_net111(netInput, netOutput, 0)  ;
  step3[i].nnu_pa = netOutput[0]        ;
  step3[i].nnu_pb = netOutput[1]        ;
  step3[i].nnu_pc = netOutput[2]        ;
 }

pred_a = pred_b = pred_c = 0 ;
fh = fe = fq = fd = fr = 0   ;
for(i = 0 ; i <  size_prot[0] ; i++)
 {
  pa = (step3[i].lpa + step3[i].nnb_pa + step3[i].nnu_pa) /3. ;
  pb = (step3[i].lpb + step3[i].nnb_pb + step3[i].nnu_pb) /3. ;
  pc = (step3[i].lpc + step3[i].nnb_pc + step3[i].nnu_pc) /3. ;
  if(pa > pb && pa > pc) pred_a++;
  if(pb > pa && pb > pc) pred_b++;
  if(pc > pa && pc > pb) pred_c++;
  if(i_seq[0][i] ==  6) fh++ ;
  if(i_seq[0][i] ==  3) fe++ ;
  if(i_seq[0][i] == 13) fq++ ;
  if(i_seq[0][i] ==  2) fd++ ;
  if(i_seq[0][i] == 14) fr++ ;
 }
frac_alpha = (float) pred_a / (float)  size_prot[0]    ;
frac_beta  = (float) pred_b / (float)  size_prot[0]    ;
frac_h     = (float) fh     / (float)  size_prot[0]    ;
frac_e     = (float) fe     / (float)  size_prot[0]    ;
frac_q     = (float) fq     / (float)  size_prot[0]    ;
frac_d     = (float) fd     / (float)  size_prot[0]    ;
frac_r     = (float) fr     / (float)  size_prot[0]    ;

for(i=0 ; i < size_prot[0] ; i++)
 {
  aa0 = 0. ; ba0 = 0 ; ab0 = 0 ; bb0 = 0;
  k = 0 ;
  for(m=i-3 ; m <= i+3; m++)
   {
   if(m >= 0 && m < size_prot[0])
    {
    aa0 += coeff_hydro[i_seq[0][m]]*sin(k*1.7453) ;
    ba0 += coeff_hydro[i_seq[0][m]]*cos(k*1.7453) ;
    ab0 += coeff_hydro[i_seq[0][m]]*sin(k*3.1416) ;
    bb0 += coeff_hydro[i_seq[0][m]]*cos(k*3.1416) ;
    k++ ;
    }
   }
   hydro_alpha[i] = sqrt(aa0*aa0 + ba0*ba0)/ (float) k ;
   hydro_beta [i] = sqrt(ab0*ab0 + bb0*bb0)/ (float) k ;
  }

MLLC(name_step3, char, MAX_FIC)                    ;
strncpy(name_step3,"", MAX_FIC)                    ;
if(mode == 0) truncate_name(file_seq , name_step3) ;
else          truncate_name(file_clu , name_step3) ;
strcat(name_step3, ".all_step3")                   ;
if ((fp=fopen(name_step3, "w")) == NULL)
       {
       printf("Unable to open file for writing in function net_line_step3 ... \n") ;
       exit(1);
       }
for(i = 0 ; i < size_prot[0] ; i++)
 {
  fprintf(fp, "%f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f\n",
               step3[i].lpa    , step3[i].lpb    , step3[i].lpc    ,
               step3[i].nnb_pa , step3[i].nnb_pb , step3[i].nnb_pc ,
               step3[i].nnu_pa , step3[i].nnu_pb , step3[i].nnu_pc ,
               hydro_alpha[i]  , hydro_beta[i]   , frac_alpha      ,
               frac_beta       , frac_h          , frac_e          ,
               frac_q          , frac_d          , frac_r          ) ;
 }
fclose(fp) ;

if(check_pred == 1)
 {
  for(i = 0 ; i < size_prot[0] ; i++)
   {
    voix[i].step3_a = voix[i].step3_b = voix[i].step3_c = 0 ;
    v[0] = vote(step3[i].lpa    , step3[i].lpb    , step3[i].lpc   )    ;
    v[1] = vote(step3[i].nnb_pa , step3[i].nnb_pb , step3[i].nnb_pc)    ;
    v[2] = vote(step3[i].nnu_pa , step3[i].nnu_pb , step3[i].nnu_pc)    ;
    for(k=0 ; k < 3 ; k++)
     {
      if(v[k] == 'a') voix[i].step3_a++ ;
      if(v[k] == 'b') voix[i].step3_b++ ;
      if(v[k] == 'c') voix[i].step3_c++ ;
     }
   }
 }



free(netInput)    ;
free(netOutput)   ;
free(hydro_alpha) ;
free(hydro_beta)  ;
free(step3)       ;
for(i = 0; i < 24 ; i++) free(attrib[i]) ; free(attrib) ;

if(verbose == 1) printf("neural networks step 3 and subsequent computations are computed ...\n") ;

return ;
}
