/************************************************************************/
/*                            Prof v1.0                                 */
/*                     Author : Mohammed OUALI                          */
/*  A full description of the method is given in the paper entitled     */
/* "Cascaded Mutiple Classifers for Secondary Structure Prediction"     */
/*              by Mohammed OUALI and Ross D. KING                      */
/*                 (Submitted for publication)                          */
/*                                                                      */
/*   Copyright (C) 1999 Mohammed OUALI and University of Wales          */
/*             (Aberystwyth) Penglais, Aberystwyth                      */
/*                 Ceredigion SY23 3DB WALES                            */
/*                        U.K.                                          */
/*                                                                      */
/* This software can be copied and used freely providing it is not      */
/* resold in any form and its use is acknowledged.                      */
/* This software is provided by ``as is'' and any express or implied    */
/* warranties, including, but not limited to, the implied warranties of */
/* merchantability and fitness for a particular purpose are disclaimed. */
/* In no event shall the regents or contributors be liable for any      */
/* direct, indirect, incidental, special, exemplary, or consequential   */
/* damages (including, but not limited to, procurement of substitute    */
/* goods or services; loss of use, data, or profits; or business        */
/* interruption) however caused and on any theory of liability, whether */
/* in contract, strict liability, or tort (including negligence or      */
/* otherwise) arising in any way out of the use of this software, even  */
/* if advised of the possibility of such damage.                        */
/*                                                                      */
/************************************************************************/
#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include "Prof.h"
#include "gor.h"

void quadratic_gor(void)
{
int i , j , k , u ;
float c_a , c_b , c_c ;
float norm ;
float dxa , dxb , dxc ;
float attrib[14] ;
float va[14], vb[14], vc[14] ;

float *pla , *plb ;

/**/
float vect_a[14] =
 {3.382, 14.070, -1.830, -2.042,  1.238,  2.851,  2.385,  5.895,  5.088,  5.684, -0.907, -2.275,  1.911, 14.039}  ;
/**/
float vect_b[14] =
 {-0.452, -0.174,  0.429,  0.938,  7.274,  4.264, -1.284, -5.732,  5.909,  3.842,  2.113,  0.071, -0.221,  0.081} ;
/**/
float vect_c[14] =
 {-1.963, -8.567,  1.054,  0.777, -3.742, -4.067,  0.926,  2.870, -5.577, -4.842, -0.529,  1.765, -0.615, -8.702} ;
/**/
float quad_mat_a[14][14] =
{
{ 27.271, -25.816,-29.627,  38.544,  -0.257, -15.486,  11.093,  -1.278,  -3.554,  -2.621, -2.211,  -1.340, -5.415,  -4.444},
{-25.816,-140.317, 38.803, 154.783, -18.283, -31.440,   0.019, -30.281, -10.177,  -2.170,  4.197,   7.578, -2.385,  -6.884},
{-29.627,  38.803, 50.120, -90.908, -17.617,  70.825,  -1.140,  -8.375,   6.080,  -7.359, -3.517,   1.961, -1.527,   0.146},
{ 38.544, 154.783,-90.908,-338.771,  70.132, 215.972, -19.140,  -1.199,  12.697, -29.316,-13.085,  -7.470,  4.816,   7.745},
{ -0.257, -18.282,-17.617,  70.132,  30.680,-125.354,   3.690,  93.966, -17.243, -22.704,  7.358,  -1.950, -6.725,  -0.414},
{-15.486, -31.440, 70.824, 215.972,-125.354,-398.794, 108.305, 244.969, -45.503, -20.168, 14.655, -21.526,-11.918,  -3.585},
{ 11.093,   0.019, -1.140, -19.139,   3.689, 108.305, -37.083,-181.848,  36.118, 116.894,-14.347, -29.500, 12.983,  -0.571},
{ -1.279, -30.281, -8.375,  -1.199,  93.966, 244.969,-181.849,-433.180, 123.117, 243.454,-39.055, -17.519, 11.816, -21.754},
{ -3.553, -10.176,  6.079,  12.697, -17.243, -45.502,  36.118, 123.117, -26.974,-159.235, 11.987,  91.350, -7.499, -25.592},
{ -2.621,  -2.170, -7.359, -29.316, -22.703, -20.167, 116.894, 243.454,-159.235,-380.041, 96.550, 206.143,-29.296, -34.612},
{ -2.211,   4.197, -3.517, -13.085,   7.358,  14.654, -14.347, -39.055,  11.987,  96.550, 20.272, -99.096,-16.856,  38.935},
{ -1.340,   7.578,  1.961,  -7.470,  -1.950, -21.526, -29.500, -17.519,  91.350, 206.143,-99.096,-303.514, 41.878, 140.966},
{ -5.415,  -2.385, -1.527,   4.816,  -6.724, -11.918,  12.983,  11.816,  -7.499, -29.296,-16.856,  41.878, 24.419, -22.742},
{ -4.444,  -6.884,  0.146,   7.745,  -0.414,  -3.585,  -0.572, -21.754, -25.592, -34.612, 38.935, 140.966,-22.742,-132.970},
} ;
/**/
float quad_mat_b[14][14] =
{
{-48.183, -0.360, 33.100,   9.678,   3.503,  -4.759,   2.842,   3.386,  -1.073,   2.022,  4.789,   4.940, -5.777, -5.907},
{ -0.360, 55.045, -5.724, -88.140,   1.976,  35.977,   8.045,  12.718,  -1.462, -11.647, -0.853,   3.959, -3.481, -2.117},
{ 33.100, -5.724,-98.075, -17.205,  66.486,  29.949,  -0.958, -12.362,   2.194,   5.169, -9.071,  -6.529,  4.903,  3.130},
{  9.678,-88.140,-17.205, 197.286,  10.204,-145.716,  -3.550,  28.036,  -1.854,  13.377,  3.230, -14.599,  0.634,  5.010},
{  3.503,  1.976, 66.486,  10.204,-168.148, -56.083,  97.074,  48.832,  -9.706, -26.406,  4.297,  17.482, -2.024,-10.293},
{ -4.759, 35.977, 29.949,-145.716, -56.083, 224.660,  30.042,-140.927,  -2.685,  23.633, -8.370,  13.709,  6.608,-12.573},
{  2.842,  8.045, -0.958,  -3.550,  97.074,  30.043,-190.073, -65.623,  87.375,  43.684,  1.503, -19.414,  3.046, 17.848},
{  3.386, 12.718,-12.362,  28.036,  48.832,-140.927, -65.622, 228.874,  25.409,-134.127,  5.801,  28.439, -1.408, 11.213},
{ -1.073, -1.462,  2.194,  -1.854,  -9.706,  -2.686,  87.375,  25.409,-139.676, -36.273, 45.386,   2.799,  6.252,  2.581},
{  2.022,-11.647,  5.169,  13.377, -26.406,  23.632,  43.684,-134.126, -36.273, 209.102,  4.241,-131.833,  1.787, 31.279},
{  4.789, -0.853, -9.071,   3.230,   4.297,  -8.370,   1.503,   5.801,  45.387,   4.241,-79.214,   2.721, 26.458,-13.500},
{  4.940,  3.959, -6.529, -14.599,  17.482,  13.709, -19.415,  28.439,   2.800,-131.833,  2.721, 173.000,  0.024,-74.783},
{ -5.777, -3.481,  4.903,   0.634,  -2.024,   6.608,   3.046,  -1.408,   6.252,   1.787, 26.458,   0.024,-42.842,  4.624},
{ -5.907, -2.117,  3.130,   5.010, -10.293, -12.573,  17.848,  11.213,   2.581,  31.279,-13.500, -74.783,  4.624, 48.501},
} ;
/**/
float quad_mat_c[14][14] =
{
{ -5.402, 15.989, 12.619, -30.510,  -2.193,  14.505,  -9.895,  -2.888,   3.005,  -0.093,  -0.584,  -0.417,  8.200,  6.435}, 
{ 15.989, 47.202,-21.657, -31.633,  11.095,  -6.309,  -5.921,   8.577,   7.828,  10.933,  -2.191,  -7.499,  4.109,  4.059},
{ 12.619,-21.657, -4.359,  69.124, -11.637, -65.064,  -0.070,  15.203,  -4.912,   0.531,   6.816,   1.454, -1.096, -1.269},
{-30.510,-31.633, 69.124,  53.057, -51.832, -17.116,  17.736, -21.451,  -9.346,   7.936,   5.990,  17.845, -3.314, -8.444},
{ -2.193, 11.095,-11.637, -51.832,  40.072, 111.476, -40.025, -88.135,  15.530,  31.865,  -6.217, -11.357,  7.187,  6.206},
{ 14.506, -6.309,-65.064, -17.116, 111.475,  44.260, -85.011, -22.382,  32.437, -13.027,  -3.946,   5.319,  3.364, 12.049},
{ -9.895, -5.922, -0.070,  17.736, -40.025, -85.012, 101.730, 150.483, -65.574,-101.081,   8.187,  33.133,-11.262,-10.621},
{ -2.888,  8.577, 15.203, -21.451, -88.135, -22.382, 150.482,  47.707, -94.774, -30.182,  23.121, -17.056, -8.249,  7.244},
{  3.005,  7.828, -4.912,  -9.346,  15.530,  32.437, -65.574, -94.774,  78.818, 125.696, -27.246, -62.283,  1.300, 16.070},
{ -0.093, 10.932,  0.531,   7.936,  31.865, -13.027,-101.081, -30.182, 125.696,  55.818, -68.554, -27.053, 19.795, -1.257},
{ -0.584, -2.191,  6.816,   5.990,  -6.217,  -3.946,   8.188,  23.121, -27.246, -68.555,  14.439,  65.032,  3.444,-16.658},
{ -0.417, -7.499,  1.454,  17.845, -11.357,   5.318,  33.134, -17.056, -62.282, -27.053,  65.031,  63.633,-29.185,-38.817},
{  8.200,  4.109, -1.096,  -3.314,   7.188,   3.364, -11.262,  -8.249,   1.300,  19.795,   3.444, -29.185, -4.443, 11.040},
{  6.435,  4.060, -1.269,  -8.444,   6.206,  12.049, -10.621,   7.244,  16.070,  -1.257, -16.658, -38.817, 11.040, 49.335},
} ;
/**/
void rematch(float *x) ;

c_a = -5.873 ; c_b = -5.082 ; c_c =  5.634 ;

/**/

MLLC(pla,  float, (size_prot[0]+6)) ;
MLLC(plb,  float, (size_prot[0]+6)) ;
MLLC(pa_q, float, length_query) ;
MLLC(pb_q, float, length_query) ;
MLLC(pc_q, float, length_query) ;

if(verbose == 1) printf("quadratic gor is started ...\n") ;

j = 3 ;
for(i = 0 ; i < length_query ; i++)
 {
  if(seq[0][i] == '-' || seq[0][i] == '.') continue ;
  else
   {
    pla[j] = pa_l[i] ;
    plb[j] = pb_l[i] ;
    j++ ;
   }
 }
for(i = 0 ; i < 3 ; i++)
 {
  pla[i] = plb[i] = 0. ;
 }
for(i = size_prot[0]+3 ; i < size_prot[0]+6 ; i++)
 {
  pla[i] = plb[i] = 0. ;
 }

u = 0 ;
for(i = 3 ; i < size_prot[0]+3 ; i++)
 {
   attrib[0]  = pla[i-3] ;
   attrib[1]  = plb[i-3] ;
   attrib[2]  = pla[i-2] ;
   attrib[3]  = plb[i-2] ;
   attrib[4]  = pla[i-1] ;
   attrib[5]  = plb[i-1] ;
   attrib[6]  = pla[i]   ;
   attrib[7]  = plb[i]   ;
   attrib[8]  = pla[i+1] ;
   attrib[9]  = plb[i+1] ;
   attrib[10] = pla[i+2] ;
   attrib[11] = plb[i+2] ;
   attrib[12] = pla[i+3] ;
   attrib[13] = plb[i+3] ;
  
  dxa = dxb = dxc = 0. ;
  for(j = 0 ; j < 14 ; j++)
   {
    va[j] = 0. ; vb[j] = 0. ; vc[j] =0. ;
    for(k = 0 ; k < 14 ; k++)
     {
      va[j] += attrib[k]*quad_mat_a[k][j] ;
      vb[j] += attrib[k]*quad_mat_b[k][j] ;
      vc[j] += attrib[k]*quad_mat_c[k][j] ;
     }
   }
  for(j = 0 ; j < 14 ; j++)
   {
    dxa += va[j]*attrib[j] ;
    dxb += vb[j]*attrib[j] ;
    dxc += vc[j]*attrib[j] ;
   }
  dxa *= 0.5 ; dxb *= 0.5 ; dxc *= 0.5 ;
  for(j =0 ; j < 14 ; j++)
   {
    dxa +=  vect_a[j]* attrib[j] ;
    dxb +=  vect_b[j]* attrib[j] ;
    dxc +=  vect_c[j]* attrib[j] ;
   }
  dxa += c_a ;
  dxb += c_b ;
  dxc += c_c ;
  pa_q[u] = 1./(1.+exp(-dxa)) ;
  pb_q[u] = 1./(1.+exp(-dxb)) ;
  pc_q[u] = 1./(1.+exp(-dxc)) ;
  norm = pa_q[u] + pb_q[u] + pc_q[u] ;
  pa_q[u] /= norm ; pb_q[u] /= norm ; pc_q[u] /= norm ;
  u++ ;
 }

rematch(pa_q) ;
rematch(pb_q) ;
rematch(pc_q) ;
/*
for(i = 0 ; i < length_query ; i++)
 {
  printf("%c %f %f %f \n", seq[0][i], pa_q[i], pb_q[i], pc_q[i]) ;
 }
*/

free(pla)  ;
free(plb)  ;

if(verbose == 1) printf("quadratic gor completed ...\n") ;

return ;
}

void   rematch(float *x )
{
int j, k    ;
float *temp ;

  temp = (float *) malloc((length_query+1)*sizeof(float)) ;

  k = 0 ;
  for(j = 0 ; j < length_query ; j++)
   {
    if(seq[0][j] != '-' && seq[0][j] != '.')
     {
      temp[j]=x[k] ;
      k++             ;
     }
    else
     {
     temp[j] = 0. ;
     }
   }
 for(j = 0 ; j < length_query ; j++)
  {
   x[j] = temp[j] ;
  }

 free(temp) ;

return ;
}

